//========================================================
//========================================================
// D E F I N I T I O N S
//========================================================
//========================================================
#define ADC0 							0
#define ADC1 							1
#define ADC2 							2
#define ADC3 							3
#define ADC4 							4
#define ADC5 							5
#define ADC6 							6				// ONLY ATmega32!
#define ADC7 							7				// ONLY ATmega32!

// PRESCALER [ADCSRA]
#define ADC_DIVISION_FACTOR_2 			0b00000001 		
#define ADC_DIVISION_FACTOR_4 			0b00000010 		
#define ADC_DIVISION_FACTOR_8 			0b00000011 		/* 1MHz / 8 = 125kHz */
#define ADC_DIVISION_FACTOR_16 			0b00000100
#define ADC_DIVISION_FACTOR_32 			0b00000101
#define ADC_DIVISION_FACTOR_64 			0b00000110
#define ADC_DIVISION_FACTOR_128 		0b00000111 		/* 16MHz / 128 = 125kHz */ 
// ADC_CLK: 50kHz~200kHz

// ADCSRA REGISTER
#define ADC_ENABLE						0b10000000		// ADEN-bit	
#define ADC_START_A2D					0b01000000		// ADSC-bit
#define ADC_AUTO_TRIGGER				0b00100000		// ADATE-bit
#define ADC_INTERRUPT_FLAG				0b00010000		// ADIF-bit
#define ADC_INTERRUPT_ENABLE			0b00001000		// ADIE-bit

// SFIOR REGISTER
#define ADC_TRIGGER_SOURCE_FREE_RUN		0b00000000		// FREE RUNNING MODE
#define ADC_TRIGGER_SOURCE_ANA_COMP		0b00000001		// ANALOG COMPARATOR
#define ADC_TRIGGER_SOURCE_EXT_INT0		0b00000010		// EXTERNAL INTERRUPT REQUEST 0
#define ADC_TRIGGER_SOURCE_T0_COMPM		0b00000011		// TIMER0 COMPARE MATCH
#define ADC_TRIGGER_SOURCE_T0_OVERF		0b00000100		// TIMER0 OVERFLOW
#define ADC_TRIGGER_SOURCE_T1_COMPB		0b00000101		// TIMER1 COMPARE MATCH B
#define ADC_TRIGGER_SOURCE_T1_OVERF		0b00000110		// TIMER1 OVERFLOW
#define ADC_TRIGGER_SOURCE_T1_CAPTE		0b00000111		// TIMER1 CAPTURE EVENT



//========================================================
//========================================================
// M A I N   A D C   F U N C T I O N S
//========================================================
//========================================================

void ADC_INITIALIZE(void)
{
	ADMUX	|=	(1<<REFS0);							/* VCC AS VOLTAGE REFERENCE */				
	
	ADCSRA	|=	ADC_ENABLE|							/* ENABLE ADC */
				ADC_DIVISION_FACTOR_128;			/* SETUP ADC CLOCK */
}

uint16_t ADC_READ_CHANNEL(uint8_t ADCn)
{
	ADCn 	&= 	7;									/* ALLOWABLE ADCn VALUES: 0,1,2,3,4,5,6,7 */
	
	ADMUX	=	(ADMUX & 0xF8)|ADCn;				/* CLEAR THE 3 BITS */
	
	ADCSRA	|=	ADC_START_A2D;						/* START CONVERSION */
	
	while(ADCSRA & ADC_START_A2D);					/* CHECK FOR COMPLETION */
	
	asm volatile("NOP");
	
	return(ADC);									/* STORE THE RESULT */
}
