/* 4-BIT MODE: ONLY ONE PORT IS USED
 * FOR COMMUNICATION WITH THE MODULE */
//#define FOSC 16000000UL

// G L O B A L   V A R S
volatile uint8_t position;

//========================================================
//========================================================
// D E F I N I T I O N S
//========================================================
//========================================================
#define	LCD_OUT					PORTD
#define	LCD_IN					PIND
#define	LCD_DDR					DDRD

// CONTROL BITS
#define	LCDRS					7 			// Def. value: 4	LCDPIN-04 > MCUPIN-06 [ATmega88]
#define	LCDRW					5 			// Def. value: 5	LCDPIN-05 > MCUPIN-11 [ATmega88]
#define	LCDEN					6 			// Def. value: 6	LCDPIN-06 > MCUPIN-12 [ATmega88]

// DATA BUS
#define	D4						0			// Def. value: 0	LCDPIN-11 > MCUPIN-02 [ATmega88]
#define	D5						1			// Def. value: 1	LCDPIN-12 > MCUPIN-03 [ATmega88]
#define	D6						2			// Def. value: 2	LCDPIN-13 > MCUPIN-04 [ATmega88]
#define	D7						3 			// Def. value: 3	LCDPIN-14 > MCUPIN-05 [ATmega88]

// REGISTER SELECT
#define	DATA					0			// DATA REGISTER
#define	CMD						1			// COMMAND REGISTER

// DDRAM ADDRESSES
#define LCD_L1      			0x80        // LCD Line 1
#define LCD_L2      			0xC0        // LCD Line 2
#define LCD_L3      			0x94        // LCD Line 3
#define LCD_L4      			0xD4    	// LCD Line 4

//========================================================
//========================================================
// P R O T O T Y P E S
//========================================================
//========================================================
void LCD_INITIALIZE(void);									// INITIALIZES THE MODULE IN 4-BIT MODE
void LCD_READ_BUSYFLAG(void);								// CHECK IF MODULE IS READY TO RECIEVE INSTRUCTIONS
void LCD_SEND_NYBBLE(uint8_t,uint8_t);						// WRITE 4 BITS TO COMMAND OR DATA REGISTER 
void LCD_SEND_BYTE(uint8_t,uint8_t);						// SPLIT BYTE INTO HI AND LO SEGMENTS AND SEND
uint8_t LCD_READ_NYBBLE(uint8_t);							// READ NYBBLE FROM MODULE
uint8_t LCD_READ_BYTE(uint8_t);								// SPLIT BYTE INTO HI AND LO SEGMENTS AND READ
void LCD_PRINT_STRING(char*);								// WRITE A STRING OF CHARACTERS TO THE MODULE
void LCD_SEND_INTEGER(uint16_t);							// WRITE AN INTEGER DIRECTLY TO THE MODULE (MAX: 9999)
void LCD_CLEAR_SCREEN(void);								// WIPE THE SCREEN (ALL PIXELS CLEARED)
void LCD_GOTO_XY(uint8_t,uint8_t);							// PLACE CURSOR AT SPECIFIED ROW AND COLUMN ON MODULE
void DELAY(void);											// DELAY ROUTINE TO REPLACE THE STANDARD DELAY FUNCTION


//========================================================
//========================================================
// M A I N   L C D   F U N C T I O N S
//========================================================
//========================================================
void DELAY(void) 
{
	int i,j,k;
    for (i=0; i<250; i++) 
    {
		k=0;
		for (j=0; j<250; j++) 
		{
			k++;
        }
    }
}

void LCD_INITIALIZE(void)
{
	position = 0;												
	
	LCD_DDR = 0b11101111;										// KEEP Pn4 AS AN INPUT

	_delay_ms(20);												// MIN. 15 MILLISECONDS POWER-ON DELAY
	//DELAY();
	//DELAY();
	
	LCD_SEND_NYBBLE(CMD, 0x03);									// SETUP IN 4-BIT MODE
	_delay_ms(20);
	//DELAY();
	LCD_SEND_NYBBLE(CMD, 0x03);
	_delay_ms(2);
	//DELAY();
	LCD_SEND_NYBBLE(CMD, 0x03);
	_delay_ms(2);
	//DELAY();
	LCD_SEND_NYBBLE(CMD, 0x02);
	_delay_ms(2);
	//DELAY();
	
	LCD_SEND_BYTE(CMD, 0x28);									// FUNCTION SET
	_delay_ms(2);
	//DELAY();
	
	LCD_SEND_BYTE(CMD, 0x08);									// DISPLAY OFF
	_delay_ms(2);
	//DELAY();
	
	LCD_SEND_BYTE(CMD, 0x01);									// CLEAR AND RETURN HOME (0,0)
	_delay_ms(2);
	//DELAY();
	
	LCD_SEND_BYTE(CMD, 0x0C); 									// NON-BLINKING CURSOR
	//lcdSendByte(CMD, 0x0F);   								// BLINKING CURSOR
	_delay_ms(2);
	//DELAY();
}

void LCD_READ_BUSYFLAG(void)
{
	while((LCD_READ_BYTE(CMD) & 0x80) == 0x80);
}

void LCD_SEND_NYBBLE(uint8_t REGISTER_SELECT, uint8_t NYBBLE)
{
	LCD_OUT = (1<<LCDEN);

	LCD_OUT |= NYBBLE;											// SEND NYBBLE
	
	if(REGISTER_SELECT == DATA)									// CHECK IF REGISTER_SELECT == DATA
	{
		LCD_OUT |= (1<<LCDRS);									// IF SO: SET THE RS BIT
	}
	else
	{
		LCD_OUT &= ~(1<<LCDRS);									// IF NOT: CLEAR THE RS BIT
	}
	
	LCD_OUT &= ~(1<<LCDEN);										// TOGGLE THE ENABLE BIT TO LATCH THE NYBBLE
}

void LCD_SEND_BYTE(uint8_t REGISTER_SELECT, uint8_t BYTE)
{
	uint8_t HINYBBLE = BYTE >> 4;								// SPLIT THE BYTE IN TO TWO NYBBLES
	uint8_t LONYBBLE = BYTE & 0x0F;
	
	if(REGISTER_SELECT == DATA && position == 20)
	{
		LCD_SEND_BYTE(CMD, 0xC0);
	}
	else if(REGISTER_SELECT == DATA && position == 40)
	{
		LCD_SEND_BYTE(CMD, 0x94);
	}
	else if(REGISTER_SELECT == DATA && position == 60)
	{
		LCD_SEND_BYTE(CMD, 0xD4);
	}
	
	LCD_READ_BUSYFLAG();
	
	LCD_SEND_NYBBLE(REGISTER_SELECT, HINYBBLE);					// WRITE THE HI NYBBLE 
	
	LCD_SEND_NYBBLE(REGISTER_SELECT, LONYBBLE);					// WRITE THE LO NYBBLE
	
	if(REGISTER_SELECT == DATA && ++position == 80) position = 0;
}

uint8_t LCD_READ_NYBBLE(uint8_t REGISTER_SELECT)
{
	uint8_t NYBBLE = 0x00;
	
	LCD_DDR = 0b11100000;										// KEEP Pn4 AS AN INPUT
	
	LCD_OUT |= (1<<LCDEN) | (1<<LCDRW);
	
	if(REGISTER_SELECT == DATA)
	{
		LCD_OUT |= (1<<LCDRS);
	}
	else
	{
		LCD_OUT &= ~(1<<LCDRS);
	}
	
	asm volatile("nop\n\t"::);									// ALLOW A FEW CLOCKS TO SETTLE THE BIT
	asm volatile("nop\n\t"::);	
	
	NYBBLE = LCD_IN & 0b00001111;								// READ DATA BUS AND MASK THE LOWER 4 NYBBLE
	
	LCD_OUT &= ~(1<<LCDEN);
	
	LCD_OUT &= ~(1<<LCDRW);
	
	LCD_DDR = 0b11101111;										// KEEP Pn4 AS AN INPUT
	
	return NYBBLE;							
}

uint8_t LCD_READ_BYTE(uint8_t REGISTER_SELECT)
{
	uint8_t BYTE = 0x00;
	uint8_t HINYBBLE = 0x00;
	uint8_t LONYBBLE = 0x00;
	
	HINYBBLE = LCD_READ_NYBBLE(REGISTER_SELECT);				// READ HI AND LO NYBBLE
	LONYBBLE = LCD_READ_NYBBLE(REGISTER_SELECT);
	
	BYTE = (HINYBBLE << 4) | LONYBBLE;							// RE-ASSEMBLE AS 1 BYTE
	
	return BYTE;
}

void LCD_PRINT_STRING(char *STRINGPOINTER)					// DEFAULT: uint8_t* STRINGPOINTER
{
	while (*STRINGPOINTER) LCD_SEND_BYTE(DATA, *STRINGPOINTER++);
}

void LCD_SEND_INTEGER(uint16_t INTEGER)
{
	uint8_t THOUSANDS = INTEGER / 1000;									// BREAK DOWN INTO THOUSANDS
	LCD_SEND_BYTE(DATA, (int)THOUSANDS + 48);
	
	uint8_t HUNDREDS = (INTEGER - THOUSANDS*1000) / 100;				// BREAK DOWN INTO HUNDREDS
	LCD_SEND_BYTE(DATA, (int)HUNDREDS + 48);
	
	uint8_t TENS = (INTEGER - THOUSANDS*1000 - HUNDREDS*100 ) / 10;		// BREAK DOWN INTO TENS
	LCD_SEND_BYTE(DATA, (int)TENS + 48);
	
	uint8_t ONES = (INTEGER - THOUSANDS*1000 - HUNDREDS*100 - TENS*10);	// BREAK DOWN INTO ONES
	LCD_SEND_BYTE(DATA, (int)ONES + 48);
}

void LCD_CLEAR_SCREEN(void)
{
	LCD_SEND_BYTE(CMD, 0x01);
	position = 0;
}

/*void LCD_GOTO_XY(uint8_t x, uint8_t y)
{
	position = (y * 20) + x;
	
	//	Send the correct commands to the command register of the LCD
	if(position < 20)
		LCD_SEND_BYTE(CMD, 0x80 | position);
	else if(position >= 20 && position < 40)
		LCD_SEND_BYTE(CMD, 0x80 | (position % 20 + 0x40));
	else if(position >= 41 && position < 60)
		LCD_SEND_BYTE(CMD, 0x80 | (position % 40 + 0x14));
	else if(position >= 20 && position < 40)
		LCD_SEND_BYTE(CMD, 0x80 | (position % 60 + 0x54));
}*/		

void LCD_GOTO_XY(uint8_t x, uint8_t y)
{
	//	Determine the new position
	position = (y * 20) + x;
	
	//	Send the correct commands to the command register of the LCD
	if(position < 20)
		LCD_SEND_BYTE(CMD, 0x80 | position);
	else if(position >= 20 && position < 40)
		LCD_SEND_BYTE(CMD, 0x80 | (position % 20 + 0x40));
	else if(position >= 41 && position < 60)
		LCD_SEND_BYTE(CMD,0x80 | (position % 40 + 0x14));
	else if(position >= 20 && position < 80)	// <--- default value: < 60
		LCD_SEND_BYTE(CMD, 0x80 | (position % 60 + 0x54));
}
