/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This library contains a collection of useful functions 	*
 * for controlling the timer functionality on the ATmega88	*
 * microprocessor.											*
 * 															*
 * This library is compatible with ATmega48P, ATmega168P 	*
 * and ATmega328P.											*
 * 															*
 * Feel free to edit the code to suite your particular		*
 * needs if you dont use any of the above mentioned MPUs.	*
 * 															*
 * - J. Hostrup | 2015										*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
  
#include <inttypes.h>

//========================================================
//========================================================
// D E F I N I T I O N S
//========================================================
//========================================================

//TIFR0 DEFINITIONS [ F L A G S ] [ATmega88P]
#define T0_COMPARE_A_FLAG		 	0x02
#define T0_COMPARE_B_FLAG		 	0x04
#define T0_OVERFLOW_FLAG			0x01

//TIFR1 DEFINITIONS [ F L A G S ] [ATmega88P]
#define T1_INPUT_CAPTURE_FLAG		0x20
#define T1_COMPARE_A_FLAG			0x10
#define T1_COMPARE_B_FLAG			0x08
#define T1_OVERFLOW_FLAG			0x04

#define T2_COMPARE_FLAG				0x80
#define T2_OVERFLOW_FLAG			0x40

//========================================================
//========================================================
// P R O T O T Y P E S
//========================================================
//========================================================
void TIMER0_RESET();											// DISABLE TIMER0
void TIMER0_INITIALIZE();										// INITIALIZE TIMER0 | 8-BIT
void TIMER0_WGM_MODE_NORMAL();									// WAVEFORM GENERATION MODE 0
void TIMER0_WGM_MODE_PWM_PHASE_CORRECT();						// WAVEFORM GENERATION MODE 1
void TIMER0_WGM_MODE_CTC();										// WAVEFORM GENERATION MODE 2
void TIMER0_WGM_MODE_FAST_PWM_0xFF();							// WAVEFORM GENERATION MODE 3
void TIMER0_WGM_MODE_PWM_PHASE_CORRECT_0CRA();					// WAVEFORM GENERATION MODE 5
void TIMER0_WGM_MODE_FAST_PWM_OCRA();							// WAVEFORM GENERATION MODE 7
void TIMER0_CMO_MODE_CHANNEL_A_NORMAL();						// COMPARE MATCH OUTPUT MODE PIN OC0A
void TIMER0_CMO_MODE_CHANNEL_A_TOGGLE();						// COMPARE MATCH OUTPUT MODE PIN OC0A
void TIMER0_CMO_MODE_CHANNEL_A_CLEAR();							// COMPARE MATCH OUTPUT MODE PIN OC0A
void TIMER0_CMO_MODE_CHANNEL_A_SET();							// COMPARE MATCH OUTPUT MODE PIN OC0A
void TIMER0_CMO_MODE_CHANNEL_B_NORMAL();						// COMPARE MATCH OUTPUT MODE PIN OC0B
void TIMER0_CMO_MODE_CHANNEL_B_TOGGLE();						// COMPARE MATCH OUTPUT MODE PIN OC0B
void TIMER0_CMO_MODE_CHANNEL_B_CLEAR();							// COMPARE MATCH OUTPUT MODE PIN OC0B
void TIMER0_CMO_MODE_CHANNEL_B_SET();							// COMPARE MATCH OUTPUT MODE PIN OC0B
void TIMER0_PRESCALE_OFF_TIMER_STOPPED();
void TIMER0_PRESCALE_1();
void TIMER0_PRESCALE_8();
void TIMER0_PRESCALE_64();
void TIMER0_PRESCALE_256();
void TIMER0_PRESCALE_1024();
void TIMER0_PRESCALE_CLK_ON_FALLING_EDGE_PINT0();				// EXTERNAL CLOCK SOURCE
void TIMER0_PRESCALE_CLK_ON_RISING_EDGE_PINT0();				// EXTERNAL CLOCK SOURCE
void TIMER0_OVERFLOW_INTERRUPT_ENABLE();						// INTERRUPT ENABLE
void TIMER0_COMPARE_MATCH_A_INTERRUPT_ENABLE();					// INTERRUPT ENABLE
void TIMER0_COMPARE_MATCH_B_INTERRUPT_ENABLE();					// INTERRUPT ENABLE
void TIMER1_RESET();											// DISABLE TIMER1
void TIMER1_INITIALIZE();										// INITIALIZE TIMER1 | 16-BIT
void TIMER1_WGM_MODE_NORMAL();									// WAVEFORM GENERATION MODE 0
void TIMER1_WGM_MODE_PWM_PHASE_CORRECT_8BIT();					// WAVEFORM GENERATION MODE 1
void TIMER1_WGM_MODE_PWM_PHASE_CORRECT_9BIT();					// WAVEFORM GENERATION MODE 2
void TIMER1_WGM_MODE_PWM_PHASE_CORRECT_10BIT();					// WAVEFORM GENERATION MODE 3
void TIMER1_WGM_MODE_CTC_OCR1A();								// WAVEFORM GENERATION MODE 4
void TIMER1_WGM_MODE_FAST_PWM_8BIT();							// WAVEFORM GENERATION MODE 5
void TIMER1_WGM_MODE_FAST_PWM_9BIT();							// WAVEFORM GENERATION MODE 6
void TIMER1_WGM_MODE_FAST_PWM_10BIT();							// WAVEFORM GENERATION MODE 7
void TIMER1_WGM_MODE_PWM_PHASE_FREQ_CORRECT_ICR1();				// WAVEFORM GENERATION MODE 8
void TIMER1_WGM_MODE_PWM_PHASE_FREQ_CORRECT_OCR1A();			// WAVEFORM GENERATION MODE 9
void TIMER1_WGM_MODE_PWM_PHASE_CORRECT_ICR1();					// WAVEFORM GENERATION MODE 10
void TIMER1_WGM_MODE_PWM_PHASE_CORRECT_OCR1A();					// WAVEFORM GENERATION MODE 11
void TIMER1_WGM_MODE_CTC_ICR1();								// WAVEFORM GENERATION MODE 12
void TIMER1_WGM_MODE_FAST_PWM_ICR1();							// WAVEFORM GENERATION MODE 14
void TIMER1_WGM_MODE_FAST_PWM_OCR1A();							// WAVEFORM GENERATION MODE 15
void TIMER1_CMO_MODE_CHANNEL_A_NORMAL();						// COMPARE MATCH OUTPUT MODE PIN OC1A
void TIMER1_CMO_MODE_CHANNEL_A_TOGGLE();						// COMPARE MATCH OUTPUT MODE PIN OC1A
void TIMER1_CMO_MODE_CHANNEL_A_CLEAR();							// COMPARE MATCH OUTPUT MODE PIN OC1A
void TIMER1_CMO_MODE_CHANNEL_A_SET();							// COMPARE MATCH OUTPUT MODE PIN OC1A
void TIMER1_CMO_MODE_CHANNEL_B_NORMAL();						// COMPARE MATCH OUTPUT MODE PIN OC1B
void TIMER1_CMO_MODE_CHANNEL_B_TOGGLE();						// COMPARE MATCH OUTPUT MODE PIN OC1B
void TIMER1_CMO_MODE_CHANNEL_B_CLEAR();							// COMPARE MATCH OUTPUT MODE PIN OC1B
void TIMER1_CMO_MODE_CHANNEL_B_SET();							// COMPARE MATCH OUTPUT MODE PIN OC1B
void TIMER1_INPUT_CAPTURE_EDGE_SELECT_POSITIVE();				// INPUT CAPTURE EDGE SELECT +
void TIMER1_INPUT_CAPTURE_EDGE_SELECT_NEGATIVE();				// INPUT CAPTURE EDGE SELECT -
void TIMER1_INPUT_CAPTURE_NOISE_CANCELER();						// INPUT CAPTURE NOISE CANCELER
void TIMER1_PRESCALE_OFF_TIMER_STOPPED();
void TIMER1_PRESCALE_1();
void TIMER1_PRESCALE_8();
void TIMER1_PRESCALE_64();
void TIMER1_PRESCALE_256();
void TIMER1_PRESCALE_1024();
void TIMER1_PRESCALE_CLK_ON_FALLING_EDGE_PINT1();				// EXTERNAL CLOCK SOURCE
void TIMER1_PRESCALE_CLK_ON_RISING_EDGE_PINT1();				// EXTERNAL CLOCK SOURCE
void TIMER1_OVERFLOW_INTERRUPT_ENABLE();						// INTERRUPT ENABLE
void TIMER1_COMPARE_MATCH_A_INTERRUPT_ENABLE();					// INTERRUPT ENABLE
void TIMER1_COMPARE_MATCH_B_INTERRUPT_ENABLE();					// INTERRUPT ENABLE
void TIMER1_INPUT_CAPTURE_INTERRUPT_ENABLE();					// INTERRUPT ENABLE
void TIMER2_RESET();											// DISABLE TIMER2
void TIMER2_INITIALIZE();										// INITIALIZE TIMER2 | 8-BIT
void TIMER2_WGM_MODE_NORMAL();									// WAVEFORM GENERATION MODE 0
void TIMER2_WGM_MODE_PWM_PHASE_CORRECT_0xFF();					// WAVEFORM GENERATION MODE 1
void TIMER2_WGM_MODE_CTC();										// WAVEFORM GENERATION MODE 2
void TIMER2_WGM_MODE_FAST_PWM_0xFF();							// WAVEFORM GENERATION MODE 3
void TIMER2_WGM_MODE_PWM_PHASE_CORRECT_OCRA();					// WAVEFORM GENERATION MODE 5
void TIMER2_WGM_MODE_FAST_PWM_OCRA();							// WAVEFORM GENERATION MODE 7
void TIMER2_CMO_MODE_CHANNEL_A_NORMAL();						// COMPARE MATCH OUTPUT MODE PIN OC2A
void TIMER2_CMO_MODE_CHANNEL_A_TOGGLE();						// COMPARE MATCH OUTPUT MODE PIN OC2A
void TIMER2_CMO_MODE_CHANNEL_A_CLEAR();							// COMPARE MATCH OUTPUT MODE PIN OC2A
void TIMER2_CMO_MODE_CHANNEL_A_SET();							// COMPARE MATCH OUTPUT MODE PIN OC2A
void TIMER2_CMO_MODE_CHANNEL_B_NORMAL();						// COMPARE MATCH OUTPUT MODE PIN OC2B
void TIMER2_CMO_MODE_CHANNEL_B_TOGGLE();						// COMPARE MATCH OUTPUT MODE PIN OC2B
void TIMER2_CMO_MODE_CHANNEL_B_CLEAR();							// COMPARE MATCH OUTPUT MODE PIN OC2B
void TIMER2_CMO_MODE_CHANNEL_B_SET();							// COMPARE MATCH OUTPUT MODE PIN OC2B
void TIMER2_PRESCALE_OFF_TIMER_STOPPED();	
void TIMER2_PRESCALE_1();
void TIMER2_PRESCALE_8();
void TIMER2_PRESCALE_32();										// ONLY TIMER2!
void TIMER2_PRESCALE_64();
void TIMER2_PRESCALE_128();										// ONLY TIMER2!
void TIMER2_PRESCALE_256();
void TIMER2_PRESCALE_1024();
void TIMER2_OVERFLOW_INTERRUPT_ENABLE();						// INTERRUPT ENABLE
void TIMER2_COMPARE_MATCH_A_INTERRUPT_ENABLE();					// INTERRUPT ENABLE
void TIMER2_COMPARE_MATCH_B_INTERRUPT_ENABLE();					// INTERRUPT ENABLE




//========================================================
//========================================================
// TIMER0 MODES
//========================================================
//========================================================
void TIMER0_RESET()
{
	TCCR0A		=		0x00;
	TCCR0B		=		0x00;
}

void TIMER0_WGM_MODE_NORMAL()
{
	TCCR0A		|=		(0<<WGM01) | (0<<WGM00);
	TCCR0B		|=		(0<<WGM02);
}

void TIMER0_WGM_MODE_PWM_PHASE_CORRECT()
{
	TCCR0A		|=		(0<<WGM01) | (1<<WGM00);
	TCCR0B		|=		(0<<WGM02);
}

void TIMER0_WGM_MODE_CTC()
{
	TCCR0A		|=		(1<<WGM01) | (0<<WGM00);
	TCCR0B		|=		(0<<WGM02);
}

void TIMER0_WGM_MODE_FAST_PWM_0xFF()
{
	TCCR0A		|=		(1<<WGM01) | (1<<WGM00);
	TCCR0B		|=		(0<<WGM02);
}

void TIMER0_WGM_MODE_PWM_PHASE_CORRECT_0CRA()
{
	TCCR0A		|=		(0<<WGM01) | (1<<WGM00);
	TCCR0B		|=		(1<<WGM02);
}

void TIMER0_WGM_MODE_FAST_PWM_OCRA()
{
	TCCR0A		|=		(1<<WGM01) | (1<<WGM00);
	TCCR0B		|=		(1<<WGM02);
}

void TIMER0_CMO_MODE_CHANNEL_A_NORMAL()
{
	TCCR0A 		|=		(0<<COM0A1)|(0<<COM0A0);
}

void TIMER0_CMO_MODE_CHANNEL_A_TOGGLE()
{
	TCCR0A 		|=		(0<<COM0A1)|(1<<COM0A0);
}						

void TIMER0_CMO_MODE_CHANNEL_A_CLEAR()						
{
	TCCR0A 		|=		(1<<COM0A1)|(0<<COM0A0);	
}

void TIMER0_CMO_MODE_CHANNEL_A_SET()							
{
	TCCR0A 		|=		(1<<COM0A1)|(1<<COM0A0);	
}

void TIMER0_CMO_MODE_CHANNEL_B_NORMAL()						
{
	TCCR0A 		|=		(0<<COM0B1)|(0<<COM0B0);	
}

void TIMER0_CMO_MODE_CHANNEL_B_TOGGLE()						
{
	TCCR0A 		|=		(0<<COM0B1)|(1<<COM0B0);
}

void TIMER0_CMO_MODE_CHANNEL_B_CLEAR()							
{
	TCCR0A 		|=		(1<<COM0B1)|(0<<COM0B0);
}

void TIMER0_CMO_MODE_CHANNEL_B_SET()							
{
	TCCR0A 		|=		(1<<COM0B1)|(1<<COM0B0);
}
//========================================================
//========================================================
// TIMER0 PRESCALER
//========================================================
//========================================================
void TIMER0_PRESCALE_OFF_TIMER_STOPPED()
{
	TCCR0B 		|=		( (0<<CS02) | (0<<CS01) | (0<<CS00) );
}

void TIMER0_PRESCALE_1()
{
	TCCR0B 		|=		( (0<<CS02) | (0<<CS01) | (1<<CS00) );
}

void TIMER0_PRESCALE_8()
{
	TCCR0B 		|=		( (0<<CS02) | (1<<CS01) | (0<<CS00) );
}

void TIMER0_PRESCALE_64()
{
	TCCR0B 		|=		( (0<<CS02) | (1<<CS01) | (1<<CS00) );
}

void TIMER0_PRESCALE_256()
{
	TCCR0B 		|=		( (1<<CS02) | (0<<CS01) | (0<<CS00) );
}

void TIMER0_PRESCALE_1024()
{
	TCCR0B 		|=		( (1<<CS02) | (0<<CS01) | (1<<CS00) );
}

void TIMER0_PRESCALE_CLK_ON_FALLING_EDGE_PINT0()
{
	TCCR0B 		|=		( (1<<CS02) | (1<<CS01) | (0<<CS00) );
}

void TIMER0_PRESCALE_CLK_ON_RISING_EDGE_PINT0()
{
	TCCR0B 		|=		( (1<<CS02) | (1<<CS01) | (1<<CS00) );
}
//========================================================
//========================================================
// TIMER0 INTERRUPTS
//========================================================
//========================================================
void TIMER0_OVERFLOW_INTERRUPT_ENABLE()
{
	TIMSK0 		|= 		(1<<TOIE0);
}

void TIMER0_COMPARE_MATCH_A_INTERRUPT_ENABLE()
{
	TIMSK0 		|= 		(1<<OCIE0A);
}

void TIMER0_COMPARE_MATCH_B_INTERRUPT_ENABLE()
{
	TIMSK0 		|= 		(1<<OCIE0B);
}
//========================================================
//========================================================
// TIMER1 MODES
//========================================================
//========================================================
void TIMER1_RESET()
{
	TCCR1A		=		0x00;
	TCCR1B		=		0x00;
}

void TIMER1_WGM_MODE_NORMAL()
{
	TCCR1A		|=		( (0<<WGM11) | (0<<WGM10) );
	TCCR1B		|=		( (0<<WGM13) | (0<<WGM12) );
}

void TIMER1_WGM_MODE_PWM_PHASE_CORRECT_8BIT()
{
	TCCR1A		|=		( (0<<WGM11) | (1<<WGM10) );
	TCCR1B		|=		( (0<<WGM13) | (0<<WGM12) );
}

void TIMER1_WGM_MODE_PWM_PHASE_CORRECT_9BIT()
{
	TCCR1A		|=		( (1<<WGM11) | (0<<WGM10) );
	TCCR1B		|=		( (0<<WGM13) | (0<<WGM12) );
}

void TIMER1_WGM_MODE_PWM_PHASE_CORRECT_10BIT()
{
	TCCR1A		|=		( (1<<WGM11) | (1<<WGM10) );
	TCCR1B		|=		( (0<<WGM13) | (0<<WGM12) );
}

void TIMER1_WGM_MODE_CTC_OCR1A()
{
	TCCR1A		|=		( (0<<WGM11) | (0<<WGM10) );
	TCCR1B		|=		( (0<<WGM13) | (1<<WGM12) );
}

void TIMER1_WGM_MODE_FAST_PWM_8BIT()
{
	TCCR1A		|=		( (0<<WGM11) | (1<<WGM10) );
	TCCR1B		|=		( (0<<WGM13) | (1<<WGM12) );
}

void TIMER1_WGM_MODE_FAST_PWM_9BIT()
{
	TCCR1A		|=		( (1<<WGM11) | (0<<WGM10) );
	TCCR1B		|=		( (0<<WGM13) | (1<<WGM12) );
}

void TIMER1_WGM_MODE_FAST_PWM_10BIT()
{
	TCCR1A		|=		( (1<<WGM11) | (1<<WGM10) );
	TCCR1B		|=		( (0<<WGM13) | (1<<WGM12) );
}

void TIMER1_WGM_MODE_PWM_PHASE_FREQ_CORRECT_ICR1()
{
	TCCR1A		|=		( (0<<WGM11) | (0<<WGM10) );
	TCCR1B		|=		( (1<<WGM13) | (0<<WGM12) );
}

void TIMER1_WGM_MODE_PWM_PHASE_FREQ_CORRECT_OCR1A()
{
	TCCR1A		|=		( (0<<WGM11) | (1<<WGM10) );
	TCCR1B		|=		( (1<<WGM13) | (0<<WGM12) );
}

void TIMER1_WGM_MODE_PWM_PHASE_CORRECT_ICR1()
{
	TCCR1A		|=		( (1<<WGM11) | (0<<WGM10) );
	TCCR1B		|=		( (1<<WGM13) | (0<<WGM12) );
}

void TIMER1_WGM_MODE_PWM_PHASE_CORRECT_OCR1A()
{
	TCCR1A		|=		( (1<<WGM11) | (1<<WGM10) );
	TCCR1B		|=		( (1<<WGM13) | (0<<WGM12) );
}

void TIMER1_WGM_MODE_CTC_ICR1()
{
	TCCR1A		|=		( (0<<WGM11) | (0<<WGM10) );
	TCCR1B		|=		( (1<<WGM13) | (1<<WGM12) );
}

void TIMER1_WGM_MODE_FAST_PWM_ICR1()
{
	TCCR1A		|=		( (1<<WGM11) | (0<<WGM10) );
	TCCR1B		|=		( (1<<WGM13) | (1<<WGM12) );
}

void TIMER1_WGM_MODE_FAST_PWM_OCR1A()
{
	TCCR1A		|=		( (1<<WGM11) | (1<<WGM10) );
	TCCR1B		|=		( (1<<WGM13) | (1<<WGM12) );
}

void TIMER1_INPUT_CAPTURE_EDGE_SELECT_POSITIVE()
{
	TCCR1B		|=		(1<<ICES1);
}

void TIMER1_INPUT_CAPTURE_EDGE_SELECT_NEGATIVE()
{
	TCCR1B		|=		(0<<ICES1);
}

void TIMER1_INPUT_CAPTURE_NOISE_CANCELER()
{
	TCCR1B		|=		(1<<ICNC1);
}

void TIMER1_CMO_MODE_CHANNEL_A_NORMAL()
{
	TCCR1A		|=		(0<<COM1A1)|(0<<COM1A0);
}

void TIMER1_CMO_MODE_CHANNEL_A_TOGGLE()
{
	TCCR1A		|=		(0<<COM1A1)|(1<<COM1A0);
}

void TIMER1_CMO_MODE_CHANNEL_A_CLEAR()
{
	TCCR1A		|=		(1<<COM1A1)|(0<<COM1A0);
}

void TIMER1_CMO_MODE_CHANNEL_A_SET()
{
	TCCR1A		|=		(1<<COM1A1)|(1<<COM1A0);
}

void TIMER1_CMO_MODE_CHANNEL_B_NORMAL()
{
	TCCR1A		|=		(0<<COM1B1)|(0<<COM1B0);
}

void TIMER1_CMO_MODE_CHANNEL_B_TOGGLE()
{
	TCCR1A		|=		(0<<COM1B1)|(1<<COM1B0);
}

void TIMER1_CMO_MODE_CHANNEL_B_CLEAR()
{
	TCCR1A		|=		(1<<COM1B1)|(0<<COM1B0);
}

void TIMER1_CMO_MODE_CHANNEL_B_SET()
{
	TCCR1A		|=		(1<<COM1B1)|(1<<COM1B0);
}
//========================================================
//========================================================
// TIMER1 PRESCALER
//========================================================
//========================================================
void TIMER1_PRESCALE_OFF_TIMER_STOPPED()
{
	TCCR1B 		|=		( (0<<CS12) | (0<<CS11) | (0<<CS10) );
}

void TIMER1_PRESCALE_1()
{
	TCCR1B 		|=		( (0<<CS12) | (0<<CS11) | (1<<CS10) );
}

void TIMER1_PRESCALE_8()
{
	TCCR1B 		|=		( (0<<CS12) | (1<<CS11) | (0<<CS10) );
}

void TIMER1_PRESCALE_64()
{
	TCCR1B 		|=		( (0<<CS12) | (1<<CS11) | (1<<CS10) );
}

void TIMER1_PRESCALE_256()
{
	TCCR1B 		|=		( (1<<CS12) | (0<<CS11) | (0<<CS10) );
}

void TIMER1_PRESCALE_1024()
{
	TCCR1B 		|=		( (1<<CS12) | (0<<CS11) | (1<<CS10) );
}

void TIMER1_PRESCALE_CLK_ON_FALLING_EDGE_PINT1()
{
	TCCR1B 		|=		( (1<<CS12) | (1<<CS11) | (0<<CS10) );
}

void TIMER1_PRESCALE_CLK_ON_RISING_EDGE_PINT1()
{
	TCCR1B 		|=		( (1<<CS12) | (1<<CS11) | (1<<CS10) );
}
//========================================================
//========================================================
// TIMER1 INTERRUPTS
//========================================================
//========================================================
void TIMER1_OVERFLOW_INTERRUPT_ENABLE()
{
	TIMSK1 		|= 		( (1<<TOIE1) );
}

void TIMER1_COMPARE_MATCH_A_INTERRUPT_ENABLE()
{
	TIMSK1 		|= 		( (1<<OCIE1A) );
}

void TIMER1_COMPARE_MATCH_B_INTERRUPT_ENABLE()
{
	TIMSK1 		|= 		( (1<<OCIE1B) );
}

void TIMER1_INPUT_CAPTURE_INTERRUPT_ENABLE()
{
	TIMSK1		|=		(1<<ICIE1);
}
//========================================================
//========================================================
// TIMER2 MODES
//========================================================
//========================================================
void TIMER2_RESET()
{
	TCCR2A		=		0x00;
	TCCR2B		=		0x00;
}

void TIMER2_WGM_MODE_NORMAL()
{
	TCCR2A		|=		( (0<<WGM11) | (0<<WGM10) );
	TCCR2B		|=		(0<<WGM22);
}

void TIMER2_WGM_MODE_PWM_PHASE_CORRECT_0xFF()
{
	TCCR2A		|=		( (0<<WGM11) | (1<<WGM10) );
	TCCR2B		|=		(0<<WGM22);
}

void TIMER2_WGM_MODE_CTC()
{
	TCCR2A		|=		( (1<<WGM11) | (0<<WGM10) );
	TCCR2B		|=		(0<<WGM22);
}

void TIMER2_WGM_MODE_FAST_PWM_0xFF()
{
	TCCR2A		|=		( (1<<WGM11) | (1<<WGM10) );
	TCCR2B		|=		(0<<WGM22);
}

void TIMER2_WGM_MODE_PWM_PHASE_CORRECT_OCRA()
{
	TCCR2A		|=		( (0<<WGM11) | (1<<WGM10) );
	TCCR2B		|=		(1<<WGM22);
}

void TIMER2_WGM_MODE_FAST_PWM_OCRA()
{
	TCCR2A		|=		( (1<<WGM11) | (1<<WGM10) );
	TCCR2B		|=		(1<<WGM22);
}

void TIMER2_CMO_MODE_CHANNEL_A_NORMAL()
{
	TCCR2A		|=		(0<<COM2A1)|(0<<COM2A0);
}

void TIMER2_CMO_MODE_CHANNEL_A_TOGGLE()						
{
	TCCR2A		|=		(0<<COM2A1)|(1<<COM2A0);
}

void TIMER2_CMO_MODE_CHANNEL_A_CLEAR()							
{
	TCCR2A		|=		(1<<COM2A1)|(0<<COM2A0);
}

void TIMER2_CMO_MODE_CHANNEL_A_SET()							
{
	TCCR2A		|=		(1<<COM2A1)|(1<<COM2A0);
}

void TIMER2_CMO_MODE_CHANNEL_B_NORMAL()						
{
	TCCR2A		|=		(0<<COM2B1)|(0<<COM2B0);
}

void TIMER2_CMO_MODE_CHANNEL_B_TOGGLE()						
{
	TCCR2A		|=		(0<<COM2B1)|(1<<COM2B0);
}

void TIMER2_CMO_MODE_CHANNEL_B_CLEAR()							
{
	TCCR2A		|=		(1<<COM2B1)|(0<<COM2B0);
}

void TIMER2_CMO_MODE_CHANNEL_B_SET()							
{
	TCCR2A		|=		(1<<COM2B1)|(1<<COM2B0);
}
//========================================================
//========================================================
// TIMER2 PRESCALER
//========================================================
//========================================================
void TIMER2_PRESCALE_OFF_TIMER_STOPPED()
{
	TCCR2B 		|=		( (0<<CS22) | (0<<CS21) | (0<<CS20) );
}

void TIMER2_PRESCALE_1()
{
	TCCR2B 		|=		( (0<<CS22) | (0<<CS21) | (1<<CS20) );
}

void TIMER2_PRESCALE_8()
{
	TCCR2B 		|=		( (0<<CS22) | (1<<CS21) | (0<<CS20) );
}

void TIMER2_PRESCALE_32()
{
	TCCR2B 		|=		( (0<<CS22) | (1<<CS21) | (1<<CS20) );
}

void TIMER2_PRESCALE_64()
{
	TCCR2B 		|=		( (1<<CS22) | (0<<CS21) | (0<<CS20) );
}

void TIMER2_PRESCALE_128()
{
	TCCR2B 		|=		( (1<<CS22) | (0<<CS21) | (1<<CS20) );
}

void TIMER2_PRESCALE_256()
{
	TCCR2B 		|=		( (1<<CS22) | (1<<CS21) | (0<<CS20) );
}

void TIMER2_PRESCALE_1024()
{
	TCCR2B 		|=		( (1<<CS22) | (1<<CS21) | (1<<CS20) );
}
//========================================================
//========================================================
// TIMER2 INTERRUPTS
//========================================================
//========================================================
void TIMER2_OVERFLOW_INTERRUPT_ENABLE()
{
	TIMSK2 		|= 		( (1<<TOIE2) );
}

void TIMER2_COMPARE_MATCH_A_INTERRUPT_ENABLE()
{
	TIMSK2 		|= 		( (1<<OCIE2A) );
}

void TIMER2_COMPARE_MATCH_B_INTERRUPT_ENABLE()
{
	TIMSK2 		|= 		( (1<<OCIE2B) );
}

//===============================================================
//===============================================================
// I N I T I A L I Z E   T I M E R / C O U N T E R 0   (8-BIT)
//===============================================================
//===============================================================
void TIMER0_INITIALIZE()
{
	TIMER0_RESET();
	TIMER0_WGM_MODE_NORMAL();					
	TIMER0_CMO_MODE_CHANNEL_A_NORMAL();
	TIMER0_CMO_MODE_CHANNEL_B_NORMAL();
	TIMER0_OVERFLOW_INTERRUPT_ENABLE();
//	TIMER0_COMPARE_MATCH_A_INTERRUPT_ENABLE();
	TIMER0_PRESCALE_CLK_ON_RISING_EDGE_PINT0();
}

//===============================================================
//===============================================================
// I N I T I A L I Z E   T I M E R / C O U N T E R 1   (16-BIT)
//===============================================================
//===============================================================
void TIMER1_INITIALIZE()
{
	// define sample window lenght
	OCR1A = 15625;
	
	TIMER1_RESET();
	TIMER1_WGM_MODE_CTC_OCR1A();								// CLEAR TIMER1 ON COMPARE WITH OCR1A VALUE
	TIMER1_CMO_MODE_CHANNEL_A_NORMAL();						
	TIMER1_CMO_MODE_CHANNEL_B_NORMAL();
	TIMER1_PRESCALE_1024();										// 16MHz / 1024 = 15625 Hz, T = 0.000064 s
//	TIMER1_INPUT_CAPTURE_EDGE_SELECT_POSITIVE();				
//	TIMER1_INPUT_CAPTURE_NOISE_CANCELER();
//	TIMER1_INPUT_CAPTURE_INTERRUPT_ENABLE();
	TIMER1_COMPARE_MATCH_A_INTERRUPT_ENABLE();					// EXECUTE ISR(TIMER1_COMPA_vect) ON COMPARE MATCH
//	TIMER1_OVERFLOW_INTERRUPT_ENABLE();
}

//===============================================================
//===============================================================
// I N I T I A L I Z E   T I M E R / C O U N T E R 2   (8-BIT)
//===============================================================
//===============================================================
void TIMER2_INITIALIZE()
{
	TIMER2_WGM_MODE_NORMAL();
	TIMER2_PRESCALE_64();
}
